//========================================================================================
//
//	File:				FWEventH.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWEVENTH_H
#define FWEVENTH_H

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CPoint;
class FW_CNullEvent;
class FW_CMouseEvent;
class FW_CEmbeddedMouseEvent;
class FW_CBorderMouseEvent;
class FW_CVirtualKeyEvent;
class FW_CCharKeyEvent;
class FW_CActivateEvent;
class FW_CSuspendResumeEvent;
class FW_CMenuEvent;
class FW_CMacWindowEvent;
class FW_CMenuBar;
class ODFrame;
class ODFacet;
class FW_MEventHandler;

//========================================================================================
//	 typedef
//========================================================================================

typedef unsigned long FW_Priority;

const FW_Priority kNoPriority = 1000;		

//========================================================================================
//	 class FW_MEventHandler
//========================================================================================

class FW_MEventHandler
{
public:
	FW_DECLARE_CLASS

	typedef FW_MEventHandler* FW_MEventHandlerPtr;
	typedef ODID EventHandlerId;

//----------------------------------------------------------------------------------------
//	Constructors/Destructor
//
protected:
	FW_MEventHandler();
	FW_MEventHandler(Environment* ev, FW_MEventHandler* nextEventHandler, FW_Priority thePriority);

public:
	virtual ~ FW_MEventHandler();

//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_MEventHandler*			GetNextEventHandler(Environment* ev) const;
	void						SetNextEventHandler(Environment* ev, FW_MEventHandler* nextEventHandler);
		
	virtual FW_Priority			GetPriority(Environment* ev) const;
	virtual void				SetPriority(Environment* ev, FW_Priority thePriority);
	
	FW_Boolean					IsEnabled(Environment* ev) const;
	virtual void				Enable(Environment* ev);
	virtual void				Disable(Environment* ev);

	// ----- Event handler management -----

	virtual void 				AdoptEventHandler(Environment* ev, FW_MEventHandler* eventHandler);
	virtual FW_MEventHandler* 	RemoveEventHandler(Environment* ev, FW_MEventHandler* eventHandler);

	// ----- Event Handling Protocal -----

	virtual FW_Boolean 			HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	virtual FW_Boolean 			DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	
	virtual FW_Boolean 			HandleMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean			DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);

	virtual FW_Boolean 			HandleBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean 			DoBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	
	virtual FW_Boolean 			HandleMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean 			DoMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	
	virtual FW_Boolean 			HandleMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Boolean 			DoMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);

	virtual FW_Boolean 			HandleBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Boolean 			DoBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);

	virtual FW_Boolean 			HandleMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Boolean 			DoMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	
	virtual FW_Boolean 			HandleMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);
	virtual FW_Boolean 			DoMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);

	virtual FW_Boolean 			HandleVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	virtual FW_Boolean 			DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);

	virtual FW_Boolean 			HandleCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);
	virtual FW_Boolean 			DoCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);

	virtual FW_Boolean 			HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	virtual FW_Boolean 			DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

	virtual FW_Boolean 			HandleAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Boolean 			DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	
	// ----- Target management -----

	virtual FW_MEventHandler* 	GetTarget(Environment* ev) const;
	virtual void 				SetTarget(Environment* ev, FW_MEventHandler* theTarget);
	FW_Boolean 					IsTarget(Environment* ev);
	FW_Boolean 					BecomeTarget(Environment* ev, FW_Boolean tabSelection = FALSE);
	FW_Boolean 					ResignTarget(Environment* ev);
	virtual FW_Boolean 			WantsToBeTarget(Environment* ev);
	virtual long 				WantsToResignTarget(Environment* ev);
	virtual void 				ActivateTarget(Environment* ev, FW_Boolean tabSelection = FALSE);
	virtual void 				DeactivateTarget(Environment* ev);

	//------------------------------------------------------------------------------------
	// Conversion utilities
	//------------------------------------------------------------------------------------
	
	static FW_MEventHandler*	ODFrameToEventHandler(Environment* ev, ODFrame* theFrame);

//------------------------------------------------------------------------------------
//	Internal use Only
//
public:
	// ----- Dispatching -----
	FW_Boolean			DispatchMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Boolean			DispatchBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Boolean			DispatchMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Boolean 			DispatchIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	FW_Boolean 			DispatchMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Boolean 			DispatchBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Boolean 			DispatchMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Boolean 			DispatchMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);
	FW_Boolean 			DispatchVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	FW_Boolean 			DispatchCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);
	FW_Boolean 			DispatchMenuEvent(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	FW_Boolean 			DispatchAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);

protected:
	virtual void 		CreateEventHandlers(Environment* ev);
	
private:
	void				PrivInsert(Environment* ev, FW_MEventHandler* eventHandler);
		
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_MEventHandler*							fNextEventHandler;
	FW_Boolean									fEnabled;
	FW_Priority									fPriority;
	FW_TOrderedCollection<FW_MEventHandler>* 	fEventHandlers;
};

//========================================================================================
//	 Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::GetNextEventHandler()
//----------------------------------------------------------------------------------------
inline FW_MEventHandler* FW_MEventHandler::GetNextEventHandler(Environment*) const
{
	return fNextEventHandler;
}

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::SetNextEventHandler()
//----------------------------------------------------------------------------------------
inline void FW_MEventHandler::SetNextEventHandler(Environment*, FW_MEventHandler* nextEventHandler)
{
	fNextEventHandler = nextEventHandler;
}

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::IsEnabled()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_MEventHandler::IsEnabled(Environment*) const
{
	return fEnabled;
}

#endif
