//========================================================================================
//
//	File:				FWEventD.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWEVENTD_H
#include "FWEventD.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWODMISS_H
#include "FWODMiss.h"
#endif

#ifndef FWCURSOR_H
#include "FWCursor.h"
#endif

#ifndef SLMIXOS_H
#include "SLMixOS.h"	// for FW_Beep
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODWindow_xh
#include <Window.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__TOOLUTILS__)
#include <ToolUtils.h>
#endif

#pragma segment FWEvents

//========================================================================================
//	globals
//========================================================================================

unsigned short FW_CEventDispatcher::gAutoRepeatCount = 0;

unsigned long FW_CEventDispatcher::gLastMouseUp = 0;
FW_SPlatformPoint FW_CEventDispatcher::gLastMouseUpWhere;

short FW_CEventDispatcher::gClickCount = 1;

//========================================================================================
//	class FW_CEventDispatcher
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::FW_CEventDispatcher
//----------------------------------------------------------------------------------------

FW_CEventDispatcher::FW_CEventDispatcher() :
	fLastMouseDownTarget(NULL),
	fLastMouseDownFrame(NULL),
	fEventHandler(NULL),
	fMenuBar(NULL)
{
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::FW_CEventDispatcher
//----------------------------------------------------------------------------------------

FW_CEventDispatcher::FW_CEventDispatcher(FW_MEventHandler* theEventHandler, FW_CMenuBar* theMenuBar) :
	fLastMouseDownTarget(NULL),
	fLastMouseDownFrame(NULL),
	fEventHandler(theEventHandler),
	fMenuBar(theMenuBar)
{
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::~FW_CEventDispatcher
//----------------------------------------------------------------------------------------

FW_CEventDispatcher::~FW_CEventDispatcher()
{
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::Dispatch
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::Dispatch(Environment* ev, 
										ODEventData* event, 
										ODFrame* odFrame,  
										ODFacet* odFacet, 
										ODEventInfo* eventInfo)
{
	FW_Boolean handled = FALSE;
	
#ifdef FW_BUILD_MAC	
	switch (event->what)
#endif
#ifdef FW_BUILD_WIN	
	switch (event->message)
#endif
	{
	
#ifdef FW_BUILD_MAC	
		case nullEvent:
#endif
#ifdef FW_BUILD_WIN	
		case WM_NULL:
#endif
			handled = this->DispatchNullEvent(ev, event, odFrame);
			break;

#ifdef FW_BUILD_MAC	
		case mouseDown:
#endif
#ifdef FW_BUILD_WIN	
		case WM_LBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_RBUTTONDOWN:
#endif
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
		case kODEvtBGMouseDown:
#ifdef FW_BUILD_MAC
			handled = this->DispatchMouseDownEvent(ev, event, odFrame, odFacet, (event->what == kODEvtBGMouseDown));
#endif
#ifdef FW_BUILD_WIN
			handled = this->DispatchMouseDownEvent(ev, event, odFrame, odFacet, (event->message == kODEvtBGMouseDown));
#endif
			break;
#else
			handled = this->DispatchMouseDownEvent(ev, event, odFrame, odFacet, FALSE);
			break;
#endif

#ifdef FW_BUILD_MAC	
		case mouseUp:
#endif
#ifdef FW_BUILD_WIN	
		case WM_LBUTTONUP:
		case WM_MBUTTONUP:
		case WM_RBUTTONUP:
#endif
			handled = this->DispatchMouseUpEvent(ev, event, odFrame, odFacet);
			break;
			
#ifdef FW_BUILD_MAC
		case keyDown:
		case autoKey:
#endif
#ifdef FW_BUILD_WIN
		case WM_KEYDOWN:
#endif
			handled = this->DispatchVirtualKeyDownEvent(ev, event, odFrame);
			break;
			
#ifdef FW_BUILD_MAC
		case keyUp:
#endif
#ifdef FW_BUILD_WIN
		case WM_KEYUP:
#endif
			handled = this->DispatchVirtualKeyUpEvent(ev, event, odFrame);
			break;
						
#ifdef FW_BUILD_WIN
		case WM_CHAR:
			handled = this->DispatchCharKeyEvent(ev, event, odFrame);
			break;
#endif

#ifdef FW_BUILD_MAC
		case activateEvt:
#endif
#ifdef FW_BUILD_WIN
		case WM_ACTIVATE:
#endif
			handled = this->DispatchActivateEvent(ev, event, odFrame, odFacet);
			break;
		
		case kODEvtMenu:
			handled = this->DispatchMenuEvent(ev, event, odFrame);
			break;
			
#ifdef FW_BUILD_MAC
		case kODEvtWindow: 
			handled = this->DispatchWindowEvent(ev, event, odFacet);
			break;
		case osEvt:
			handled = this->DispatchOSEvent(ev, event, odFacet);
			break;
#endif

		case kODEvtMouseDownEmbedded:
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
		case kODEvtBGMouseDownEmbedded:
#ifdef FW_BUILD_MAC
			handled = this->DispatchEmbeddedMouseDownEvent(ev, event, odFrame, odFacet, eventInfo, (event->what == kODEvtBGMouseDownEmbedded));
#endif
#ifdef FW_BUILD_WIN
			handled = this->DispatchEmbeddedMouseDownEvent(ev, event, odFrame, odFacet, eventInfo, (event->message == kODEvtBGMouseDownEmbedded));
#endif
			break;
#else
			handled = this->DispatchEmbeddedMouseDownEvent(ev, event, odFrame, odFacet, eventInfo, FALSE);
			break;
#endif
			
		case kODEvtMouseUpEmbedded:
			handled = this->DispatchEmbeddedMouseUpEvent(ev, event, odFrame, odFacet, eventInfo);
			break;
			
		case kODEvtMouseDownBorder:
			handled = this->DispatchBorderMouseDownEvent(ev, event, odFrame, odFacet, eventInfo);
			break;
			
		case kODEvtMouseEnter:
			handled = this->DispatchMouseEnterEvent(ev, odFacet, eventInfo);
			break;
			
		case kODEvtMouseWithin:
			handled = this->DispatchMouseWithinEvent(ev, odFacet, eventInfo);
			break;
			
		case kODEvtMouseLeave:
			handled = this->DispatchMouseLeaveEvent(ev, odFacet, eventInfo);
			break;
	}

	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchNullEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchNullEvent(Environment* ev, ODEventData* event, ODFrame* odFrame)
{
	FW_CNullEvent theNullEvent(ev, event);	
	FW_MEventHandler* target = fEventHandler;	// = the part by default
	
	if (odFrame)
	{	
		FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
		FW_ASSERT(fwFrame != NULL);
		target = fwFrame->GetTarget(ev);	// [LSD] to change with an idle class
		FW_ASSERT(target != NULL);
	}
	
	return target->HandleIdle(ev, theNullEvent);
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMouseDownEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMouseDownEvent(Environment* ev, ODEventData* event, ODFrame* odFrame,  ODFacet* odFacet, FW_Boolean inBackground)
{
	// ofFacet is NULL when the user clicks outside a frame with the modal focus
	if (!odFacet) {
		if (inBackground) {
			return FALSE;	// Let process manager bring the part to the foreground
		} else {
			FW_Beep();		// Click is lost
			return TRUE;
		}
	}
	
	short numOfClicks = OnMouseDown(ev, event);
	
	FW_CMouseEvent theMouseEvent(ev, event, odFacet, FW_CMouseEvent::kMouseButtonDown, numOfClicks);
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);

	FW_CPoint where = theMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
	FW_MEventHandler* target = fwFrame->GetViewContaining(ev, odFacet, where);
	FW_ASSERT(target);

//	FW_ASSERT(fLastMouseDownTarget == NULL);
//	FW_ASSERT(fLastMouseDownFrame == NULL);
	fLastMouseDownTarget = target;
	fLastMouseDownFrame = fwFrame;
	
	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = inBackground ? target->DispatchBGMouseDown(ev, theMouseEvent)
							  : target->DispatchMouseDown(ev, theMouseEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMouseUpEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMouseUpEvent(Environment* ev, ODEventData* event, ODFrame* odFrame,  ODFacet* odFacet)
{
	// ofFacet is NULL when the user clicks outside a frame with the modal focus
	if (!odFacet) return TRUE;

	OnMouseUp(ev, event);
	
	FW_CMouseEvent theMouseEvent(ev, event, odFacet, FW_CMouseEvent::kMouseButtonUp);
	
	FW_MEventHandler* target = NULL;
	
//	FW_ASSERT(fLastMouseDownTarget != NULL);
//	FW_ASSERT(fLastMouseDownFrame != NULL);

	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);

	if (fLastMouseDownFrame == fwFrame)
		target = fLastMouseDownTarget;
	else
	{
		FW_CPoint where = theMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
		target = fwFrame->GetViewContaining(ev, odFacet, where);
	}

	fLastMouseDownTarget = NULL;
	fLastMouseDownFrame = NULL;

	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchMouseUp(ev, theMouseEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchEmbeddedMouseDownEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchEmbeddedMouseDownEvent(Environment* ev, 
																ODEventData* event, 
																ODFrame* odFrame,  
																ODFacet* odFacet, 
																ODEventInfo* eventInfo,
																FW_Boolean inBackground)
{
	short numOfClicks = OnMouseDown(ev, event);
	
	FW_Boolean wasSelected = eventInfo->embeddedFacet->IsSelected(ev);
	
	FW_CEmbeddedMouseEvent theEmbeddedMouseEvent(ev, 
												event, 
												odFacet, 
												eventInfo->embeddedFrame, 
												eventInfo->embeddedFacet, 
												FW_CMouseEvent::kMouseButtonDown,
												numOfClicks);
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);

	if (!inBackground && fwFrame->PrivActiveWindowOnMouseDown(ev, theEmbeddedMouseEvent.GetFacet(ev), TRUE))
		return TRUE;

	FW_CSelection* selection = fwFrame->GetPresentation(ev)->GetSelection(ev);
	if (selection)
		selection->UpdateSelectionOnMouseDown(ev, 
												theEmbeddedMouseEvent, 
												theEmbeddedMouseEvent.GetEmbeddedFacet(ev), 
												FALSE, 	// Not in active border
												FALSE);	// Not in Background

	FW_CPoint where = theEmbeddedMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
	FW_MEventHandler* target = fwFrame->GetViewContaining(ev, odFacet, where);
	FW_ASSERT(target);

//	FW_ASSERT(fLastMouseDownTarget == NULL);
//	FW_ASSERT(fLastMouseDownFrame == NULL);
	fLastMouseDownTarget = target;
	fLastMouseDownFrame = fwFrame;

	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = inBackground ? target->DispatchBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent)
							  : target->DispatchMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent);
			
		target = target->GetNextEventHandler(ev);
	}

//	[HLX] OpenDoc Bug ---- The mouseUp event is not going to go to embedded frames inside
//	my embedded frame. I need to unselect my embedded facet
	if (!handled && wasSelected)
		eventInfo->embeddedFacet->SetSelected(ev, FALSE);
//	[HLX] OpenDoc Bug		
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchEmbeddedMouseUpEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchEmbeddedMouseUpEvent(Environment* ev, ODEventData* event, ODFrame* odFrame,  ODFacet* odFacet, ODEventInfo* eventInfo)
{
	OnMouseUp(ev, event);
	
	FW_CEmbeddedMouseEvent theEmbeddedMouseEvent(ev, 
												event, 
												odFacet, 
												eventInfo->embeddedFrame, 
												eventInfo->embeddedFacet, 
												FW_CMouseEvent::kMouseButtonUp);
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	
	FW_MEventHandler* target = NULL;
	
//	FW_ASSERT(fLastMouseDownTarget != NULL);
//	FW_ASSERT(fLastMouseDownFrame != NULL);

	if (fLastMouseDownFrame == fwFrame)
		target = fLastMouseDownTarget;
	else
	{
		FW_CPoint where = theEmbeddedMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
		target = fwFrame->GetViewContaining(ev, odFacet, where);
	}
	
	fLastMouseDownTarget = NULL;
	fLastMouseDownFrame = NULL;

	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchBorderMouseDownEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchBorderMouseDownEvent(Environment* ev, ODEventData* event, ODFrame* odFrame,  ODFacet* odFacet, ODEventInfo* eventInfo)
{
	short numOfClicks = OnMouseDown(ev, event);
	
	FW_CBorderMouseEvent theBorderMouseEvent(ev, 
											event, 
											odFacet, 
											eventInfo->embeddedFrame, 
											eventInfo->embeddedFacet,
											FW_CMouseEvent::kMouseButtonDown,
											numOfClicks);
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);

	FW_gClosedHandCursor.Select();

	fwFrame->PrivActivateFrame(ev, theBorderMouseEvent.GetFacet(ev));

	FW_CSelection* selection =	fwFrame->GetPresentation(ev)->GetSelection(ev);
	if (selection)
		selection->UpdateSelectionOnMouseDown(ev, 
												theBorderMouseEvent, 
												theBorderMouseEvent.GetEmbeddedFacet(ev), 
												TRUE, 
												FALSE);
	
	FW_CPoint where = theBorderMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
	FW_MEventHandler* target = fwFrame->GetViewContaining(ev, odFacet, where);
	FW_ASSERT(target);

//	FW_ASSERT(fLastMouseDownTarget == NULL);
//	FW_ASSERT(fLastMouseDownFrame == NULL);
	fLastMouseDownTarget = target;
	fLastMouseDownFrame = fwFrame;

	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchVirtualKeyDownEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchVirtualKeyDownEvent(Environment* ev, ODEventData* event, ODFrame* odFrame)
{
	FW_Boolean handled = FALSE;
	
#ifdef FW_BUILD_MAC
	if (event->what == autoKey)
		gAutoRepeatCount++;
	else
		gAutoRepeatCount = 0;
		
	FW_CVirtualKeyEvent theVirtualKeyEvent(ev, event, FW_CVirtualKeyEvent::kKeyDown, gAutoRepeatCount);
#endif

#ifdef FW_BUILD_WIN
	FW_CVirtualKeyEvent theVirtualKeyEvent(ev, event, FW_CVirtualKeyEvent::kKeyDown, LOWORD(event->lParam));
#endif

	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	FW_MEventHandler* target = fwFrame->GetTarget(ev);
	FW_ASSERT(target != NULL);

	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchVirtualKey(ev, theVirtualKeyEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
#ifdef FW_BUILD_MAC
	// If the keycode isn't used try the char
	if (!handled)
		handled = this->DispatchCharKeyEvent(ev, event, odFrame);
#endif

	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchVirtualKeyUpEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchVirtualKeyUpEvent(Environment* ev, ODEventData* event, ODFrame* odFrame)
{
	FW_CVirtualKeyEvent theVirtualKeyEvent(ev, event, FW_CVirtualKeyEvent::kKeyUp);
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	FW_MEventHandler* target = fwFrame->GetTarget(ev);
	FW_ASSERT(target != NULL);
	
	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchVirtualKey(ev, theVirtualKeyEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchCharKeyEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchCharKeyEvent(Environment* ev, ODEventData* event, ODFrame* odFrame)
{
#ifdef FW_BUILD_MAC
	FW_CCharKeyEvent theCharKeyEvent(ev, event, gAutoRepeatCount);
#endif
#ifdef FW_BUILD_WIN
	FW_CCharKeyEvent theCharKeyEvent(ev, event, LOWORD(event->lParam));
#endif
	
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	FW_MEventHandler* target = fwFrame->GetTarget(ev);
	FW_ASSERT(target != NULL);
	
	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchCharKey(ev, theCharKeyEvent);
			
		target = target->GetNextEventHandler(ev);
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchActivateEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchActivateEvent(Environment* ev, ODEventData* event, ODFrame* odFrame,  ODFacet* odFacet)
{
	FW_UNUSED(odFrame);
	
#ifdef FW_BUILD_MAC
	FW_CActivateEvent theActivateEvent(ev, event, odFacet, (event->modifiers & activeFlag ) != 0);
#endif
#ifdef FW_BUILD_WIN
	FW_CActivateEvent theActivateEvent(ev, event, odFacet, (event->wParam != WA_INACTIVE));
#endif

	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	
	fwFrame->HandleActivateEvent(ev, theActivateEvent);
	return true;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMenuEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMenuEvent(Environment* ev, ODEventData* event, ODFrame* odFrame)
{
	ODCommandID theCommand;
	
#ifdef FW_BUILD_MAC
	ODMenuID menu = HiWord(event->message);
	ODMenuItemID item = LoWord(event->message);
	theCommand = fMenuBar->PrivMacGetCommand(ev, menu, item);
#endif
#ifdef FW_BUILD_WIN
	theCommand = event->wParam;
#endif

	FW_CMenuEvent theMenuEvent(ev, event, theCommand);

	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFrame);
	FW_ASSERT(fwFrame != NULL);
	FW_MEventHandler* target = fwFrame->GetTarget(ev);
	FW_ASSERT(target != NULL);
	
	FW_Boolean handled = FALSE;
	while (target != NULL && handled == FALSE)
	{
		handled = target->DispatchMenuEvent(ev, theMenuEvent);
			
		target = target->GetNextEventHandler(ev);
	}

#ifdef FW_BUILD_MAC
	::HiliteMenu(0);
#endif

	return handled;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMouseEnterEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMouseEnterEvent(Environment* ev, ODFacet* odFacet, ODEventInfo* eventInfo)
{
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(fwFrame != NULL);

	return fwFrame->HandleMouseEnter(ev, odFacet, FW_CPoint(eventInfo->where));;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMouseWithinEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMouseWithinEvent(Environment* ev, ODFacet* odFacet, ODEventInfo* eventInfo)
{
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(fwFrame != NULL);

	return fwFrame->HandleMouseWithin(ev, odFacet, FW_CPoint(eventInfo->where));;
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchMouseLeaveEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchMouseLeaveEvent(Environment* ev, ODFacet* odFacet, ODEventInfo* eventInfo)
{
FW_UNUSED(eventInfo);
	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(fwFrame != NULL);

	return fwFrame->HandleMouseLeave(ev, odFacet);;
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchWindowEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchWindowEvent(Environment* ev, ODEventData* event, ODFacet* odFacet)
{
	FW_CMacWindowEvent windowEvent(ev, event);

	FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(fwFrame != NULL);

	return fwFrame->HandleWindowEvent(ev, windowEvent);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::DispatchOSEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_CEventDispatcher::DispatchOSEvent(Environment* ev, ODEventData* event, ODFacet* odFacet)
{
	FW_Boolean handled = FALSE;
	
	if (((unsigned char) ( event->message >> 24 ) & 0x00FF) == 0x01)
	{
		const short kResumeMask = 0x01;	// High byte suspend/resume event 
	
		FW_Boolean goingToBackground = (event->message & kResumeMask) == 0;
		FW_CSuspendResumeEvent suspendResumeEvent(ev, event, odFacet, goingToBackground);
		
		FW_CFrame* fwFrame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
		FW_ASSERT(fwFrame != NULL);
			
		handled = fwFrame->HandleSuspendResumeEvent(ev, suspendResumeEvent);
	}
	
	return handled;
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::OnMouseUp
//----------------------------------------------------------------------------------------

void FW_CEventDispatcher::OnMouseUp(Environment* ev, ODEventData* event)
{
FW_UNUSED(ev);
#ifdef FW_BUILD_MAC
	gLastMouseUp = event->when;
	gLastMouseUpWhere = event->where;
#endif

#ifdef FW_BUILD_WIN
	gLastMouseUp = 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CEventDispatcher::OnMouseDown
//----------------------------------------------------------------------------------------

short FW_CEventDispatcher::OnMouseDown(Environment* ev, ODEventData* event)
{
FW_UNUSED(ev);
#ifdef FW_BUILD_MAC
	if (event->when - gLastMouseUp < ::GetDblTime())
	{
		short deltax = FW_Absolute(event->where.h - gLastMouseUpWhere.h);
		short deltay = FW_Absolute(event->where.v - gLastMouseUpWhere.v);
		if (deltax <= 4 && deltay <= 4)
			gClickCount++;
		else
			gClickCount = 1;
	}
	else
		gClickCount = 1;
#endif

#ifdef FW_BUILD_WIN
	gClickCount = 1;
#endif

	return gClickCount;
}