//========================================================================================
//
//	File:				FWEventH.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

#ifndef FWCURSOR_H
#include "FWCursor.h"
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#pragma segment fwevents

//========================================================================================
//	Template Instantiations
//========================================================================================

FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_MEventHandler)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_MEventHandler)

#ifdef FW_USE_TEMPLATE_PRAGMAS

#pragma template_access public
#pragma template FW_TOrderedCollection<FW_MEventHandler>
#pragma template FW_TOrderedCollectionIterator<FW_MEventHandler>

#endif

//========================================================================================
// class FW_MEventHandler
//========================================================================================

FW_DEFINE_CLASS_M0(FW_MEventHandler)

//----------------------------------------------------------------------------------------
// FW_MEventHandler::FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::FW_MEventHandler()
  : fNextEventHandler(NULL),
	fEnabled(true),
	fPriority(kNoPriority),
	fEventHandlers(NULL)
{
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::FW_MEventHandler(Environment* ev, FW_MEventHandler* nextEventHandler, FW_Priority thePriority)
  : fNextEventHandler(nextEventHandler),
	fEnabled(true),
	fPriority(thePriority),
	fEventHandlers(NULL)
{
FW_UNUSED(ev);
}

//----------------------------------------------------------------------------------------
//	FW_MEventHandler::~FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::~FW_MEventHandler()
{
	if (fEventHandlers)
	{
		FW_MEventHandler* eventHandler;
		while ((eventHandler = fEventHandlers->First()) != NULL)
		{
			fEventHandlers->Remove(eventHandler);
			delete eventHandler;  // adopted eventHandler belongs to this event handler
		}		

		delete fEventHandlers;
	}
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::GetPriority
//----------------------------------------------------------------------------------------

FW_Priority FW_MEventHandler::GetPriority(Environment* ev) const
{
FW_UNUSED(ev);
	return fPriority;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::SetPriority
//----------------------------------------------------------------------------------------

void FW_MEventHandler::SetPriority(Environment* ev, FW_Priority thePriority)
{
FW_UNUSED(ev);
	fPriority = thePriority;
	// [B1 Conversion] Changing the priority should change its position ????
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::CreateEventHandlers
//----------------------------------------------------------------------------------------

void FW_MEventHandler::CreateEventHandlers(Environment* ev)
{
FW_UNUSED(ev);
	FW_TOrderedCollection<FW_MEventHandler>* eventHandlers = FW_NEW(FW_TOrderedCollection<FW_MEventHandler>, ());
	fEventHandlers = eventHandlers;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Insert
//----------------------------------------------------------------------------------------

void FW_MEventHandler::PrivInsert(Environment* ev, FW_MEventHandler* eventHandler)
{
	FW_MEventHandler* before = NULL;
	
    FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
	for (FW_MEventHandler* handler = ite.First(); ite.IsNotComplete(); handler = ite.Next())
	{
		if (handler == eventHandler) {
			FW_DEBUG_MESSAGE("in FW_MEventHandler::PrivInsert, handler already inserted");
			return;
		}
		if (handler->GetPriority(ev) > eventHandler->GetPriority(ev))
		{
			before = handler;
		}
	}
	
	if (before)
		fEventHandlers->AddBefore(before, eventHandler);
	else
		fEventHandlers->AddLast(eventHandler);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::AdoptEventHandler
//----------------------------------------------------------------------------------------

void FW_MEventHandler::AdoptEventHandler(Environment* ev, FW_MEventHandler* eventHandler)
{
	if (!fEventHandlers)
		CreateEventHandlers(ev);

	PrivInsert(ev, eventHandler);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::RemoveEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::RemoveEventHandler(Environment* ev, FW_MEventHandler* eventHandler)
{
FW_UNUSED(ev);
	FW_MEventHandler* theEventHandler = NULL;
	if (fEventHandlers)
	{
		fEventHandlers->Remove(eventHandler);
		theEventHandler = eventHandler;
	}
	return theEventHandler;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Enable
//----------------------------------------------------------------------------------------

void FW_MEventHandler::Enable(Environment* ev)
{
FW_UNUSED(ev);
	fEnabled = TRUE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Disable
//----------------------------------------------------------------------------------------

void FW_MEventHandler::Disable(Environment* ev)
{
FW_UNUSED(ev);
//	FW_ASSERT(fSuperView != NULL);	// a CFrame cannot be disabled
	fEnabled = FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);		
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchIdle(ev, theNullEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleIdle(ev, theNullEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{	
	return DoIdle(ev, theNullEvent);	
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theNullEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMouseDown(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return HandleMouseDown(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{	
	return IsEnabled(ev) ? DoMouseDown(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theMouseEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchBGMouseDown(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return HandleBGMouseDown(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return IsEnabled(ev) ? DoBGMouseDown(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theMouseEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMouseUp(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return HandleMouseUp(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
FW_UNUSED(ev);
	return IsEnabled(ev) ? DoMouseUp(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theMouseEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------
//	By default just call DoMouseDown

FW_Boolean FW_MEventHandler::DoMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseDown(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchBGMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleBGMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------
//	By default just call DoBGMouseDown

FW_Boolean FW_MEventHandler::DoBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoBGMouseDown(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------
// By Default just call DoMouseUp

FW_Boolean FW_MEventHandler::DoMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseUp(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
FW_UNUSED(ev);
	return DoMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theBorderMouseEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchVirtualKey(ev, theVirtualKeyEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleVirtualKey(ev, theVirtualKeyEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
	return DoVirtualKey(ev, theVirtualKeyEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theVirtualKeyEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchCharKey(ev, theCharKeyEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleCharKey(ev, theCharKeyEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
	return DoCharKey(ev, theCharKeyEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theCharKeyEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMenuEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMenuEvent(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	if (fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if (eventHandler->DispatchMenuEvent(ev, theMenuEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMenu(ev, theMenuEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
FW_UNUSED(ev);
	return DoMenu(ev, theMenuEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
FW_UNUSED(ev);
FW_UNUSED(theMenuEvent);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchAdjustMenus
//----------------------------------------------------------------------------------------
//	Attention: behaviors for adjust menus are done AFTER the main handlers. This allows
//	someone to 'override' a handler

FW_Boolean FW_MEventHandler::DispatchAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, 
												FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	FW_Boolean handled = IsEnabled(ev) ? HandleAdjustMenus(ev, menuBar, hasMenuFocus, isRoot) : FALSE;

	if (!handled && fEventHandlers)
	{
		 FW_TOrderedCollectionIterator<FW_MEventHandler> ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = ite.First(); ite.IsNotComplete(); eventHandler = ite.Next())
		{
			if ((handled = eventHandler->DispatchAdjustMenus(ev, menuBar, hasMenuFocus, isRoot)) == TRUE)
				break;
		}
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleAdjustMenus(Environment *ev, 
												FW_CMenuBar* menuBar, 
												FW_Boolean hasMenuFocus, 
												FW_Boolean isRoot)
{
	return DoAdjustMenus(ev, menuBar, hasMenuFocus, isRoot);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, 
											FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
FW_UNUSED(ev);
FW_UNUSED(menuBar);
FW_UNUSED(hasMenuFocus);
FW_UNUSED(isRoot);

	return FALSE;	// not done handling
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::GetTarget
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::GetTarget(Environment* ev) const
{
	// This goes up the views and ends up with FW_CFrame::GetTarget to return its fTarget
	if (fNextEventHandler)
		return fNextEventHandler->GetTarget(ev);
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::SetTarget
//----------------------------------------------------------------------------------------

void FW_MEventHandler::SetTarget(Environment* ev, FW_MEventHandler* theTarget)
{
	// This goes up the views and ends up with FW_CFrame::SetTarget to set its fTarget
	if (fNextEventHandler)
		fNextEventHandler->SetTarget(ev, theTarget);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::IsTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::IsTarget(Environment* ev)
{
	return (this == GetTarget(ev));
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::BecomeTarget
//----------------------------------------------------------------------------------------
// BecomeTarget() must be called only after WantsToBeTarget() returned TRUE.
// It can fail if the current target didn't want to resign.

FW_Boolean FW_MEventHandler::BecomeTarget(Environment* ev, FW_Boolean tabSelection)
{
	// Cannot target disabled objects
	if (IsEnabled(ev) == FALSE)
		return FALSE;
	
	FW_MEventHandler* currentTarget = GetTarget(ev);
	if (this != currentTarget)
	{
		if (currentTarget->ResignTarget(ev))
		{
			SetTarget(ev, this);
			ActivateTarget(ev, tabSelection);
			return TRUE;
		}
		else
			return FALSE;
	}
	return TRUE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::ResignTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::ResignTarget(Environment* ev)
{
	if(this != GetTarget(ev))
		return TRUE;
			
	if (WantsToResignTarget(ev) == 0)
	{
		DeactivateTarget(ev);
		SetTarget(ev, fNextEventHandler);		// ??? What do we do in this case with no gApplication ???
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::WantsToBeTarget
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::WantsToBeTarget(Environment* ev)
{
FW_UNUSED(ev);
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::WantsToResignTarget
//----------------------------------------------------------------------------------------

long FW_MEventHandler::WantsToResignTarget(Environment* ev)
{
FW_UNUSED(ev);
	return 0;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::ActivateTarget
//----------------------------------------------------------------------------------------

void FW_MEventHandler::ActivateTarget(Environment* ev, FW_Boolean tabSelection)
{
FW_UNUSED(ev);
FW_UNUSED(tabSelection);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DeactivateTarget
//----------------------------------------------------------------------------------------

void FW_MEventHandler::DeactivateTarget(Environment* ev)
{
FW_UNUSED(ev);
}

