//========================================================================================
//
//	File:				FWScpCol.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSCPCOL_H
#define FWSCPCOL_H

#ifndef FWSCPTBL_H
#include "FWScptbl.h"
#endif

// ----- ODF Foundation Includes -----

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

//========================================================================================
//	class FW_CScriptableCollection
//========================================================================================

class FW_CScriptableCollection :  public FW_MScriptable									 
{
public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CScriptableCollection)
	
	// ----- Construction/Destruction -----
	
	FW_CScriptableCollection();
	
	virtual ~FW_CScriptableCollection();

	// ----- Reference Counting -----

	virtual long AcquireScriptable();
	
	virtual long ReleaseScriptable();

	// ----- Token Type -----
	
	virtual ODDescType GetTokenType() const;

	// ----- Collection -----
	
	long 								Count() const { return fRep->Count(); }

	void 								Add(FW_MScriptable* element);
	void								Remove(FW_MScriptable* element);
	
	void								AdjustMarks(Environment* ev, ODSLong newStart, ODSLong newStop);

	// ----- Relationship Descriptors -----
	
	virtual FW_CFrame*					GetFrame(Environment* ev, FW_CPart* part) const;

	// ----- Core Events -----

	virtual void 						DoAECountElements(Environment* ev,
														FW_CPart* part,
														const FW_CAppleEvent& event,
														FW_CAppleEvent& reply);

	virtual void						DoAEGetData(Environment* ev,
													FW_CPart* part,
													const FW_CAppleEvent& event,
													FW_CAppleEvent& reply);

	virtual void						DoAESetData(Environment* ev,
													FW_CPart* part,
													const FW_CAppleEvent& event,
													FW_CAppleEvent& reply);

	virtual FW_Boolean					HasProperty(ODDescType whichProperty) const;

	virtual void 						GetPropertyInfo(Environment* ev,	
														FW_CPart* part,
														FW_Boolean& canUndo,
														FW_Boolean& causesChange) const;

	virtual FW_Boolean					GetProperty(Environment* ev, 
													FW_CPart* part,
													FW_CDesc& propertyValue, 
													ODDescType whichProperty,
													ODDescType desiredType = typeWildCard) const;
												
	virtual void						SetProperty(Environment* ev, 
													FW_CPart* part,
													FW_CDesc& propertyValue, 
													ODDescType whichProperty);
													
	virtual void						RestoreProperty(Environment* ev,
														FW_CPart* part,
														FW_CDesc& propertyValue,
														ODDescType whichProperty);
												
	virtual void						GetUndoStrings(Environment* ev,
													FW_CPart* part,
													ODDescType whichProperty,
													FW_CString& undoString,
													FW_CString& redoString) const;
	// ----- Dependencies -----
	
	virtual void						HandlePrimaryDeleted(FW_MScriptable* primary);

private:
	
	long									fRefCount;
	FW_TOrderedCollection<FW_MScriptable>* 	fRep;
};

#endif